#ifndef TREE_H
#define TREE_H

#include <stdio.h>

enum tag_type { /* misc. tags */
                TAG_BOLD = 0,
                TAG_EMPHASIS,
                TAG_ITALIC,
                TAG_PLAIN,
                TAG_STRONG,
                TAG_TYPEWRITER,
                TAG_UNDERLINE,

                /* size tags */
                TAG_SIZE_0 = 10,
                TAG_SIZE_1,
                TAG_SIZE_2,
                TAG_SIZE_3,
                TAG_SIZE_4,
                TAG_SIZE_5,
                TAG_SIZE_6,
                TAG_SIZE_7,
                TAG_SIZE_8,
                TAG_SIZE_9,

                /* color tags */
                TAG_COLOR_RED = 20,
                TAG_COLOR_GREEN,
                TAG_COLOR_BLUE,
                TAG_COLOR_CYAN,
                TAG_COLOR_MAGENTA,
                TAG_COLOR_YELLOW,
                TAG_COLOR_BLACK,
                TAG_COLOR_WHITE,

                /* internal flags */
                TAG_DUMMY = 30,	/* dummy tag - ignore it */
                TAG_END};		/* returned by load_tag() to indicate tag end */

enum color_code { COLOR_DEFAULT = 0,
                  COLOR_RED,
                  COLOR_GREEN,
                  COLOR_BLUE,
                  COLOR_CYAN,
                  COLOR_MAGENTA,
                  COLOR_YELLOW,
                  COLOR_BLACK,
                  COLOR_WHITE};

typedef struct
{
	/* on/off attributes */
	char bold;
	char emphasis;
	char italic;
	char strong;
	char typewriter;

	/* other attributes */
	char underline;	/* 0-3 */
	char size;			/* 0-10,  10=default */
	enum color_code color;
} attribs;

typedef struct treenode
{
	int       is_tag;	/* 1 if it's a tag, 0 if text */
	enum tag_type  tag;		/* which tag is it */
	char     *text;		/* text, if it's a text node */
	int scratch;		/* added variable to quicken routine and eliminate recursion*/

	attribs attr;			/* node attributes */

	int child_count;	/* number of children */
	struct treenode *children[1024];	/* children */

	struct treenode *parent;	/* parental node */
} node;

extern node *root;	/* root of the tree */

void load(FILE *fp);	/* load the tree */
void cleanup(node *a);	/* deletes the tree and contents */
void out(node *a);
int scan_text(node *top);

#endif /* TREE_H */
